@@ -73,13 +73,13 @@ orbit_generate_tcstruct_name (IDL_tree node)
 	case IDLN_TYPE_OBJECT: {
 		char *typespec = orbit_cbe_get_typespec_str (node);
 
-		g_string_sprintf (tmpstr, "TC_%s", typespec);
+		g_string_printf (tmpstr, "TC_%s", typespec);
 
 		g_free (typespec);
 		}
 		break;
 	default:
-		g_string_sprintf (tmpstr, "anonTC_%d", random_id++);
+		g_string_printf (tmpstr, "anonTC_%d", random_id++);
 		break;
 	}
 
@@ -151,7 +151,8 @@ orbit_output_tcstruct_anon_subnames_array (FILE *fh, IDL_tree node, int subnames
 		fprintf (fh, "static const char * anon_subnames_array%d[] = {", subnames_id);
 
 		for (l = IDL_TYPE_UNION (node).switch_body; l; l = IDL_LIST (l).next) {
-			IDL_tree dcl;
+			IDL_tree dcl, label;
+			const char *subname;
 
 			g_assert (IDL_NODE_TYPE (IDL_LIST (l).data) == IDLN_CASE_STMT);
 
@@ -162,10 +163,17 @@ orbit_output_tcstruct_anon_subnames_array (FILE *fh, IDL_tree node, int subnames
 				  IDL_NODE_TYPE (dcl) == IDLN_TYPE_ARRAY);
 
 			if (IDL_NODE_TYPE (dcl) == IDLN_IDENT)
-				fprintf (fh, "\"%s\"", IDL_IDENT (dcl).str);
-
+				subname = IDL_IDENT (dcl).str;
 			else /* IDLN_TYPE_ARRAY */
-				fprintf (fh, "\"%s\"", IDL_IDENT (IDL_TYPE_ARRAY (dcl).ident).str);
+				subname = IDL_IDENT (IDL_TYPE_ARRAY (dcl).ident).str;
+
+			/* output the name once for each label */
+			for (label = IDL_CASE_STMT (IDL_LIST (l).data).labels;
+			     label != NULL; label = IDL_LIST (label).next) {
+				fprintf (fh, "\"%s\"", subname);
+				if (IDL_LIST (label).next)
+					fprintf (fh, ", ");
+			}
 
 			if (IDL_LIST (l).next)
 				fprintf (fh, ", ");
@@ -191,7 +199,7 @@ orbit_output_tcstruct_anon_subtypes_array (FILE     *fh,
 		if (!IDL_TYPE_STRUCT (node).member_list)
 			break;
 
-		fprintf (fh, "static const CORBA_TypeCode anon_subtypes_array%d[] = {",
+		fprintf (fh, "static ORBIT2_MAYBE_CONST CORBA_TypeCode anon_subtypes_array%d[] = {",
 			 subtypes_id);
 
 		for (l = IDL_TYPE_STRUCT (node).member_list; l; l = IDL_LIST (l).next) {
@@ -242,7 +250,7 @@ orbit_output_tcstruct_anon_subtypes_array (FILE     *fh,
 		if (!IDL_TYPE_UNION (node).switch_body)
 			break;
 
-		fprintf (fh, "static const CORBA_TypeCode anon_subtypes_array%d[] = {", subtypes_id);
+		fprintf (fh, "static ORBIT2_MAYBE_CONST CORBA_TypeCode anon_subtypes_array%d[] = {", subtypes_id);
 
 		for (l = IDL_TYPE_UNION (node).switch_body; l; l = IDL_LIST (l).next) {
 			IDL_tree  label, dcl;
@@ -285,7 +293,7 @@ orbit_output_tcstruct_anon_subtypes_array (FILE     *fh,
 
 		seqts = orbit_cbe_get_typespec (IDL_TYPE_SEQUENCE (node).simple_type_spec);
 
-		fprintf (fh, "static const CORBA_TypeCode anon_subtypes_array%d[] = ", subtypes_id);
+		fprintf (fh, "static ORBIT2_MAYBE_CONST CORBA_TypeCode anon_subtypes_array%d[] = ", subtypes_id);
 
 		switch (IDL_NODE_TYPE (seqts)) {
 		case IDLN_INTERFACE:
@@ -309,7 +317,7 @@ orbit_output_tcstruct_anon_subtypes_array (FILE     *fh,
 		break;
 	case IDLN_TYPE_ARRAY:
 	case IDLN_IDENT:
-		fprintf (fh, "static const CORBA_TypeCode anon_subtypes_array%d[] = ", subtypes_id);
+		fprintf (fh, "static ORBIT2_MAYBE_CONST CORBA_TypeCode anon_subtypes_array%d[] = ", subtypes_id);
 		fprintf (fh, "{(CORBA_TypeCode)&%s_struct};\n", substructname);
 		break;
 	default:
@@ -322,51 +330,35 @@ orbit_output_tcstruct_anon_sublabels_array (FILE     *fh,
 					    IDL_tree  node,
 					    int       sublabels_id)
 {
-	IDL_tree l;
+	IDL_tree l, label;
+	int      index = 0;
 	int      default_index = -1;
 
-	switch (IDL_NODE_TYPE (node)) {
-	case IDLN_TYPE_UNION: {
-		char *tmpstr;
-		int   index = 0;
+	if (IDL_NODE_TYPE (node) != IDLN_TYPE_UNION ||
+	    !IDL_TYPE_UNION (node).switch_body)
+		return default_index;
 
-		if (!IDL_TYPE_UNION (node).switch_body)
-			break;
+	fprintf (fh, "static ORBIT2_MAYBE_CONST CORBA_long anon_sublabels_array%d[] = {", sublabels_id);
 
-		fprintf (fh, "static const CORBA_long anon_sublabels_array%d[] = {", sublabels_id);
+	for (l = IDL_TYPE_UNION (node).switch_body; l; l = IDL_LIST (l).next)
+		for (label = IDL_CASE_STMT (IDL_LIST (l).data).labels; label;
+		     label = IDL_LIST (label).next, index++) {
 
-		tmpstr = orbit_cbe_get_typespec_str (IDL_TYPE_UNION (node).switch_type_spec);
-		for (l = IDL_TYPE_UNION (node).switch_body; l; l = IDL_LIST (l).next) {
-			IDL_tree label;
+			if (IDL_LIST (label).data) {
+				fprintf (fh, "(CORBA_long) ");
 
-			g_assert (IDL_NODE_TYPE (IDL_LIST (l).data) == IDLN_CASE_STMT);
+				orbit_cbe_write_const (fh, IDL_LIST (label).data);
 
-			for (label = IDL_CASE_STMT (IDL_LIST (l).data).labels; label;
-			     label = IDL_LIST (label).next, index++) {
-
-				if (IDL_LIST (label).data) {
-					fprintf (fh, "(CORBA_long) ");
-
-					orbit_cbe_write_const (fh, IDL_LIST (label).data);
-				}
-				else { /* default case */
-					fprintf (fh, "-1");
-					default_index = index;
-				}
-
-				if (IDL_LIST (label).next || IDL_LIST (l).next)
-					fprintf (fh, ", ");
+			} else { /* default case */
+				fprintf (fh, "-1");
+				default_index = index;
 			}
+
+			if (IDL_LIST (label).next || IDL_LIST (l).next)
+				fprintf (fh, ", ");
 		}
 
-		g_free (tmpstr);
-		fprintf (fh, "};\n");
-
-		}
-		break;
-	default:
-		break;
-	}
+	fprintf (fh, "};\n");
 
 	return default_index;
 }
@@ -475,27 +467,27 @@ orbit_output_tcstruct_name (FILE *fh, IDL_tree node, int array_gen_ctr)
 {
 	switch (IDL_NODE_TYPE (node)) {
 	case IDLN_TYPE_STRUCT:
-		fprintf (fh, "\"%s\"", IDL_IDENT (IDL_TYPE_STRUCT (node).ident).str);
+		fprintf (fh, "(char *)\"%s\"", IDL_IDENT (IDL_TYPE_STRUCT (node).ident).str);
 		break;
 	case IDLN_TYPE_UNION:
-		fprintf (fh, "\"%s\"", IDL_IDENT (IDL_TYPE_UNION (node).ident).str);
+		fprintf (fh, "(char *)\"%s\"", IDL_IDENT (IDL_TYPE_UNION (node).ident).str);
 		break;
 	case IDLN_TYPE_ENUM:
-		fprintf (fh, "\"%s\"", IDL_IDENT (IDL_TYPE_ENUM (node).ident).str);
+		fprintf (fh, "(char *)\"%s\"", IDL_IDENT (IDL_TYPE_ENUM (node).ident).str);
 		break;
 	case IDLN_INTERFACE:
 	case IDLN_FORWARD_DCL:
-		fprintf (fh, "\"%s\"", IDL_IDENT (IDL_INTERFACE (node).ident).str);
+		fprintf (fh, "(char *)\"%s\"", IDL_IDENT (IDL_INTERFACE (node).ident).str);
 		break;
 	case IDLN_EXCEPT_DCL:
-		fprintf (fh, "\"%s\"", IDL_IDENT (IDL_EXCEPT_DCL (node).ident).str);
+		fprintf (fh, "(char *)\"%s\"", IDL_IDENT (IDL_EXCEPT_DCL (node).ident).str);
 		break;
 	case IDLN_IDENT:
-		fprintf (fh, "\"%s\"", IDL_IDENT (node).str);
+		fprintf (fh, "(char *)\"%s\"", IDL_IDENT (node).str);
 		break;
 	case IDLN_TYPE_ARRAY:
 		if (!array_gen_ctr)
-			fprintf (fh, "\"%s\"", 
+			fprintf (fh, "(char *)\"%s\"", 
 				 IDL_IDENT (IDL_TYPE_ARRAY (node).ident).str);
 		else
 			fprintf (fh, "NULL");
@@ -511,27 +503,27 @@ orbit_output_tcstruct_repo_id (FILE *fh, IDL_tree node, int array_gen_ctr)
 {
 	switch (IDL_NODE_TYPE (node)) {
 	case IDLN_TYPE_STRUCT:
-		fprintf (fh, "\"%s\"", IDL_IDENT (IDL_TYPE_STRUCT (node).ident).repo_id);
+		fprintf (fh, "(char *)\"%s\"", IDL_IDENT (IDL_TYPE_STRUCT (node).ident).repo_id);
 		break;
 	case IDLN_TYPE_UNION:
-		fprintf (fh, "\"%s\"", IDL_IDENT (IDL_TYPE_UNION (node).ident).repo_id);
+		fprintf (fh, "(char *)\"%s\"", IDL_IDENT (IDL_TYPE_UNION (node).ident).repo_id);
 		break;
 	case IDLN_TYPE_ENUM:
-		fprintf (fh, "\"%s\"", IDL_IDENT (IDL_TYPE_ENUM (node).ident).repo_id);
+		fprintf (fh, "(char *)\"%s\"", IDL_IDENT (IDL_TYPE_ENUM (node).ident).repo_id);
 		break;
 	case IDLN_INTERFACE:
 	case IDLN_FORWARD_DCL:
-		fprintf (fh, "\"%s\"", IDL_IDENT (IDL_INTERFACE (node).ident).repo_id);
+		fprintf (fh, "(char *)\"%s\"", IDL_IDENT (IDL_INTERFACE (node).ident).repo_id);
 		break;
 	case IDLN_EXCEPT_DCL:
-		fprintf (fh, "\"%s\"", IDL_IDENT (IDL_EXCEPT_DCL (node).ident).repo_id);
+		fprintf (fh, "(char *)\"%s\"", IDL_IDENT (IDL_EXCEPT_DCL (node).ident).repo_id);
 		break;
 	case IDLN_IDENT:
-		fprintf (fh, "\"%s\"", IDL_IDENT (node).repo_id);
+		fprintf (fh, "(char *)\"%s\"", IDL_IDENT (node).repo_id);
 		break;
 	case IDLN_TYPE_ARRAY:
 		if (!array_gen_ctr)
-			fprintf (fh, "\"%s\"", IDL_IDENT (IDL_TYPE_ARRAY (node).ident).repo_id);
+			fprintf (fh, "(char *)\"%s\"", IDL_IDENT (IDL_TYPE_ARRAY (node).ident).repo_id);
 		else
 			fprintf (fh, "NULL");
 		break;
@@ -745,64 +737,73 @@ orbit_output_tcstruct_digits_scale (FILE *fh, IDL_tree node)
 		fprintf (fh, "0, 0");
 }
 
-static gint
-orbit_find_c_align (IDL_tree node)
+static void
+orbit_add_align (GSList **max, const char *str)
 {
-	gint c_align = 1;
+	GSList *l;
 
+	for (l = *max; l; l = l->next) {
+		if (!strcmp (l->data, str))
+			return;
+	}
+	*max = g_slist_prepend (*max, (gpointer) str);
+}
+
+static GSList *
+orbit_find_c_align (GSList *max, IDL_tree node)
+{
 	node = orbit_cbe_get_typespec (node);	
 
 	switch (IDL_NODE_TYPE (node)) {
 	case IDLN_TYPE_INTEGER:
 		switch (IDL_TYPE_INTEGER (node).f_type) {
 		case IDL_INTEGER_TYPE_SHORT:
-			c_align = ORBIT_ALIGNOF_CORBA_SHORT;
+			orbit_add_align (&max, "ORBIT_ALIGNOF_CORBA_SHORT");
 			break;
 		case IDL_INTEGER_TYPE_LONG:
-			c_align = ORBIT_ALIGNOF_CORBA_LONG;
+			orbit_add_align (&max, "ORBIT_ALIGNOF_CORBA_LONG");
 			break;
 		case IDL_INTEGER_TYPE_LONGLONG:
-			c_align = ORBIT_ALIGNOF_CORBA_LONG_LONG;
+			orbit_add_align (&max, "ORBIT_ALIGNOF_CORBA_LONG_LONG");
 			break;
 		}
 		break;
 	case IDLN_TYPE_FLOAT:
 		switch (IDL_TYPE_FLOAT (node).f_type) {
 		case IDL_FLOAT_TYPE_FLOAT:
-			c_align = ORBIT_ALIGNOF_CORBA_FLOAT;
+			orbit_add_align (&max, "ORBIT_ALIGNOF_CORBA_FLOAT");
 			break;
 		case IDL_FLOAT_TYPE_DOUBLE:
-			c_align = ORBIT_ALIGNOF_CORBA_DOUBLE;
+			orbit_add_align (&max, "ORBIT_ALIGNOF_CORBA_DOUBLE");
 			break;
 		case IDL_FLOAT_TYPE_LONGDOUBLE:
-			c_align = ORBIT_ALIGNOF_CORBA_LONG_DOUBLE;
+			orbit_add_align (&max, "ORBIT_ALIGNOF_CORBA_LONG_DOUBLE");
 			break;
 		}
 		break;
 	case IDLN_TYPE_ENUM:
-		c_align = ORBIT_ALIGNOF_CORBA_LONG;
+		orbit_add_align (&max, "ORBIT_ALIGNOF_CORBA_LONG");
 		break;
 	case IDLN_TYPE_CHAR: /* drop through */
 	case IDLN_TYPE_BOOLEAN:
 	case IDLN_TYPE_OCTET:
-		c_align = ORBIT_ALIGNOF_CORBA_CHAR;
+		orbit_add_align (&max, "ORBIT_ALIGNOF_CORBA_CHAR");
 		break;
 	case IDLN_TYPE_WIDE_CHAR:
-		c_align = ORBIT_ALIGNOF_CORBA_SHORT;
+		orbit_add_align (&max, "ORBIT_ALIGNOF_CORBA_SHORT");
 		break;
 	case IDLN_TYPE_UNION: {
 		IDL_tree l = IDL_TYPE_UNION (node).switch_body;
 
-		c_align = ORBIT_ALIGNOF_CORBA_STRUCT;
+		orbit_add_align (&max, "ORBIT_ALIGNOF_CORBA_STRUCT");
 
 		for (; l; l = IDL_LIST (l).next) {
 			IDL_tree subtype = IDL_MEMBER (IDL_CASE_STMT (
-					IDL_LIST (l).data).element_spec).type_spec;
-
-			c_align = MAX (c_align, orbit_find_c_align (subtype));
-		}
+				IDL_LIST (l).data).element_spec).type_spec;
+			max = orbit_find_c_align (max, subtype);
 		}
 		break;
+	}
 	case IDLN_EXCEPT_DCL: /* drop through */
 	case IDLN_TYPE_STRUCT: {
 		IDL_tree l = IDL_TYPE_STRUCT (node).member_list;
@@ -810,33 +811,32 @@ orbit_find_c_align (IDL_tree node)
 		for (; l; l = IDL_LIST (l).next) {
 			IDL_tree member = IDL_MEMBER (IDL_LIST (l).data).type_spec;
 
-			c_align = MAX (c_align, orbit_find_c_align (member));
-		}
+			max = orbit_find_c_align (max, member);
 		}
 		break;
+	}
 	case IDLN_TYPE_STRING: /* drop through */
 	case IDLN_TYPE_WIDE_STRING:
 	case IDLN_TYPE_OBJECT:
 	case IDLN_TYPE_TYPECODE:
 	case IDLN_FORWARD_DCL:
 	case IDLN_INTERFACE:
-		c_align = ORBIT_ALIGNOF_CORBA_POINTER;
+		orbit_add_align (&max, "ORBIT_ALIGNOF_CORBA_POINTER");
 		break;
 	case IDLN_TYPE_ARRAY: {
-		IDL_tree subtype = IDL_TYPE_DCL ( IDL_get_parent_node (node, IDLN_TYPE_DCL, NULL)
-							).type_spec;
-
-		c_align = orbit_find_c_align (subtype);
-		}
+		IDL_tree subtype = IDL_TYPE_DCL (
+			IDL_get_parent_node (node, IDLN_TYPE_DCL, NULL)).type_spec;
+		max = orbit_find_c_align (max, subtype);
 		break;
+	}
 	case IDLN_TYPE_SEQUENCE:
-		c_align = MAX (MAX (ORBIT_ALIGNOF_CORBA_STRUCT,
-				    ORBIT_ALIGNOF_CORBA_LONG),
-				    ORBIT_ALIGNOF_CORBA_POINTER);
+		orbit_add_align (&max, "ORBIT_ALIGNOF_CORBA_STRUCT");
+		orbit_add_align (&max, "ORBIT_ALIGNOF_CORBA_LONG");
+		orbit_add_align (&max, "ORBIT_ALIGNOF_CORBA_POINTER");
 		break;
 	case IDLN_TYPE_ANY:
-		c_align =  MAX (ORBIT_ALIGNOF_CORBA_STRUCT,
-			        ORBIT_ALIGNOF_CORBA_POINTER);
+		orbit_add_align (&max, "ORBIT_ALIGNOF_CORBA_STRUCT");
+		orbit_add_align (&max, "ORBIT_ALIGNOF_CORBA_POINTER");
 		break;
 	default:
 		g_error ("Can't find alignment %s\n", 
@@ -844,13 +844,42 @@ orbit_find_c_align (IDL_tree node)
 		break;
 	}
 
-	return c_align;
+	return max;
 }
 
 static void
 orbit_output_tcstruct_c_align (FILE *fh, IDL_tree node)
 {
-	fprintf (fh, "%d", orbit_find_c_align (node));
+	GSList *max;
+	GString *str = g_string_sized_new (120);
+
+	max = orbit_find_c_align (NULL, node);
+
+	if (!max)
+		g_string_append (str, "1");
+
+	else if (!max->next)
+		g_string_append (str, max->data);
+
+	else {
+		int i = 0;
+		GSList *l;
+
+		for (l = max; l; l = l->next) {
+			g_string_append (str, "MAX (");
+			g_string_append (str, l->data);
+			g_string_append (str, ", ");
+			i++;
+		}
+		
+		g_string_append (str, "1");
+		for (; i > 0; i--)
+			g_string_append_c (str, ')');
+	}
+
+	fprintf (fh, "%s", str->str);
+
+	g_string_free (str, TRUE);
 }
 
 static void
@@ -933,12 +962,13 @@ cbe_tc_generate (OIDL_C_Info  *ci,
 	if (!strncmp (tci->structname, "anon", 4))
 		fprintf (ci->fh, "static ");
 	else {
-		fprintf (ci->fh, "#ifdef ORBIT_IDL_C_IMODULE\n");
+		fprintf (ci->fh, "#ifdef ORBIT_IDL_C_IMODULE_%s\n",
+			 ci->c_base_name);
 		fprintf (ci->fh, "static\n");
 		fprintf (ci->fh, "#endif\n");
 	}
 
-	fprintf (ci->fh, "const struct CORBA_TypeCode_struct %s_struct = {\n",
+	fprintf (ci->fh, "ORBIT2_MAYBE_CONST struct CORBA_TypeCode_struct %s_struct = {\n",
 				tci->structname);
 
 	orbit_output_tcstruct_parent (ci->fh);
